@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,31 +16,46 @@
 
 package groovy.util;
 
+import groovy.lang.Closure;
+
 import java.util.Map;
 
 /**
- * @author Andres Almiray <aalmiray@users.sourceforge.com>
- */ 
+ * @author <a href="mailto:aalmiray@users.sourceforge.com">Andres Almiray</a>
+ * @author Danno Ferrin
+ */
 public abstract class AbstractFactory implements Factory {
     public boolean isLeaf() {
         return false;
     }
 
+    public boolean isHandlesNodeChildren() {
+        return false;
+    }
+
+    public void onFactoryRegistration(FactoryBuilderSupport builder, String registeredName, String group) {
+        // do nothing
+    }
+
     public boolean onHandleNodeAttributes( FactoryBuilderSupport builder, Object node,
             Map attributes ) {
         return true;
     }
 
-    public void onNodeCompleted( FactoryBuilderSupport builder, Object parent, Object node ) {
+    public boolean onNodeChildren( FactoryBuilderSupport builder, Object node, Closure childContent) {
+        return true;
+    }
 
+    public void onNodeCompleted( FactoryBuilderSupport builder, Object parent, Object node ) {
+        // do nothing
     }
 
     public void setParent( FactoryBuilderSupport builder, Object parent, Object child ) {
-
+        // do nothing
     }
 
     public void setChild( FactoryBuilderSupport builder, Object parent, Object child ) {
-
+        // do nothing
     }
 
 }
