@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2012 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,13 +16,32 @@
 package groovy.util;
 
 import groovy.lang.Binding;
-import groovy.lang.DeprecationException;
 import groovy.lang.GroovyClassLoader;
 import groovy.lang.GroovyCodeSource;
 import groovy.lang.GroovyResourceLoader;
 import groovy.lang.Script;
 
-import java.io.*;
+import org.codehaus.groovy.GroovyBugError;
+import org.codehaus.groovy.ast.ClassNode;
+import org.codehaus.groovy.ast.InnerClassNode;
+import org.codehaus.groovy.classgen.GeneratorContext;
+import org.codehaus.groovy.control.ClassNodeResolver;
+import org.codehaus.groovy.control.CompilationFailedException;
+import org.codehaus.groovy.control.CompilationUnit;
+import org.codehaus.groovy.control.CompilerConfiguration;
+import org.codehaus.groovy.control.Phases;
+import org.codehaus.groovy.control.SourceUnit;
+import org.codehaus.groovy.control.customizers.CompilationCustomizer;
+import org.codehaus.groovy.runtime.IOGroovyMethods;
+import org.codehaus.groovy.runtime.InvokerHelper;
+import org.codehaus.groovy.tools.gse.DependencyTracker;
+import org.codehaus.groovy.tools.gse.StringSetMap;
+
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.lang.ref.WeakReference;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -30,24 +49,13 @@ import java.net.URLConnection;
 import java.security.AccessController;
 import java.security.CodeSource;
 import java.security.PrivilegedAction;
+import java.util.HashMap;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
-import org.codehaus.groovy.ast.ClassNode;
-import org.codehaus.groovy.ast.InnerClassNode;
-import org.codehaus.groovy.classgen.GeneratorContext;
-import org.codehaus.groovy.control.CompilationFailedException;
-import org.codehaus.groovy.control.CompilationUnit;
-import org.codehaus.groovy.control.CompilerConfiguration;
-import org.codehaus.groovy.control.Phases;
-import org.codehaus.groovy.control.SourceUnit;
-import org.codehaus.groovy.runtime.InvokerHelper;
-import org.codehaus.groovy.runtime.DefaultGroovyMethods;
-import org.codehaus.groovy.tools.gse.DependencyTracker;
-import org.codehaus.groovy.tools.gse.StringSetMap;
-
 /**
  * Specific script engine able to reload modified scripts as well as dealing properly
  * with dependent scripts.
@@ -56,34 +64,27 @@ import org.codehaus.groovy.tools.gse.StringSetMap;
  * @author Marc Palmer
  * @author Guillaume Laforge
  * @author Jochen Theodorou
+ * @author Mattias Reichel
  */
 public class GroovyScriptEngine implements ResourceConnector {
 
     private static final ClassLoader CL_STUB = new ClassLoader() {
     };
+    private static final URL[] EMPTY_URL_ARRAY = new URL[0];
 
-    private static WeakReference<ThreadLocal<StringSetMap>> dependencyCache = new WeakReference<ThreadLocal<StringSetMap>>(null);
-
-    private static synchronized ThreadLocal<StringSetMap> getDepCache() {
-        ThreadLocal<StringSetMap> local = dependencyCache.get();
-        if (local != null) return local;
-        local = new ThreadLocal<StringSetMap>() {
-            @Override
-            protected StringSetMap initialValue() {
-                return new StringSetMap();
-            }
-        };
-        dependencyCache = new WeakReference<ThreadLocal<StringSetMap>>(local);
-        return local;
+    private static class LocalData {
+        CompilationUnit cu;
+        StringSetMap dependencyCache = new StringSetMap();
+        Map<String, String> precompiledEntries = new HashMap<String, String>();
     }
 
-    private static WeakReference<ThreadLocal<CompilationUnit>> localCu = new WeakReference<ThreadLocal<CompilationUnit>>(null);
+    private static WeakReference<ThreadLocal<LocalData>> localData = new WeakReference<ThreadLocal<LocalData>>(null);
 
-    private static synchronized ThreadLocal<CompilationUnit> getLocalCompilationUnit() {
-        ThreadLocal<CompilationUnit> local = localCu.get();
+    private static synchronized ThreadLocal<LocalData> getLocalData() {
+        ThreadLocal<LocalData> local = localData.get();
         if (local != null) return local;
-        local = new ThreadLocal<CompilationUnit>();
-        localCu = new WeakReference<ThreadLocal<CompilationUnit>>(local);
+        local = new ThreadLocal<LocalData>();
+        localData = new WeakReference<ThreadLocal<LocalData>>(local);
         return local;
     }
 
@@ -92,30 +93,44 @@ public class GroovyScriptEngine implements ResourceConnector {
     private final ClassLoader parentLoader;
     private final GroovyClassLoader groovyLoader;
     private final Map<String, ScriptCacheEntry> scriptCache = new ConcurrentHashMap<String, ScriptCacheEntry>();
-    private CompilerConfiguration config = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
+    private CompilerConfiguration config;
+
+    {
+        config = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
+        config.setSourceEncoding("UTF-8");
+    }
+
 
     //TODO: more finals?
 
     private static class ScriptCacheEntry {
         private final Class scriptClass;
-        private final long lastModified;
+        private final long lastModified, lastCheck;
         private final Set<String> dependencies;
+        private final boolean sourceNewer;
 
-        public ScriptCacheEntry(Class clazz, long modified, Set<String> depend) {
+        public ScriptCacheEntry(Class clazz, long modified, long lastCheck, Set<String> depend, boolean sourceNewer) {
             this.scriptClass = clazz;
             this.lastModified = modified;
+            this.lastCheck = lastCheck;
             this.dependencies = depend;
+            this.sourceNewer = sourceNewer;
+        }
+
+        public ScriptCacheEntry(ScriptCacheEntry old, long lastCheck, boolean sourceNewer) {
+            this(old.scriptClass, old.lastModified, lastCheck, old.dependencies, sourceNewer);
         }
     }
 
     private class ScriptClassLoader extends GroovyClassLoader {
+
+
         public ScriptClassLoader(GroovyClassLoader loader) {
             super(loader);
-            setResLoader();
         }
 
-        public ScriptClassLoader(ClassLoader loader) {
-            super(loader);
+        public ScriptClassLoader(ClassLoader loader, CompilerConfiguration config) {
+            super(loader, config, false);
             setResLoader();
         }
 
@@ -123,31 +138,34 @@ public class GroovyScriptEngine implements ResourceConnector {
             final GroovyResourceLoader rl = getResourceLoader();
             setResourceLoader(new GroovyResourceLoader() {
                 public URL loadGroovySource(String className) throws MalformedURLException {
-                	String filename;
-                	for (String extension : getConfig().getScriptExtensions()) {
+                    String filename;
+                    for (String extension : getConfig().getScriptExtensions()) {
                         filename = className.replace('.', File.separatorChar) + "." + extension;
-		                try {
-		                    URLConnection dependentScriptConn = rc.getResourceConnection(filename);
-		                    return dependentScriptConn.getURL();
-		                } catch (ResourceException e) {
-		                    //TODO: maybe do something here?
-		                }
-                	}
+                        try {
+                            URLConnection dependentScriptConn = rc.getResourceConnection(filename);
+                            return dependentScriptConn.getURL();
+                        } catch (ResourceException e) {
+                            //TODO: maybe do something here?
+                        }
+                    }
                     return rl.loadGroovySource(className);
                 }
             });
         }
 
         @Override
-        protected CompilationUnit createCompilationUnit(CompilerConfiguration config, CodeSource source) {
-            CompilationUnit cu = super.createCompilationUnit(config, source);
-            getLocalCompilationUnit().set(cu);
-            final StringSetMap cache = getDepCache().get();
+        protected CompilationUnit createCompilationUnit(CompilerConfiguration configuration, CodeSource source) {
+            CompilationUnit cu = super.createCompilationUnit(configuration, source);
+            LocalData local = getLocalData().get();
+            local.cu = cu;
+            final StringSetMap cache = local.dependencyCache;
+            final Map<String, String> precompiledEntries = local.precompiledEntries;
 
             // "." is used to transfer compilation dependencies, which will be
             // recollected later during compilation
             for (String depSourcePath : cache.get(".")) {
                 try {
+                    cache.get(depSourcePath);
                     cu.addSource(getResourceConnection(depSourcePath).getURL());
                 } catch (ResourceException e) {
                     /* ignore */
@@ -156,6 +174,7 @@ public class GroovyScriptEngine implements ResourceConnector {
 
             // remove all old entries including the "." entry
             cache.clear();
+
             cu.addPhaseOperation(new CompilationUnit.PrimaryClassNodeOperation() {
                 @Override
                 public void call(final SourceUnit source, GeneratorContext context, ClassNode classNode)
@@ -163,62 +182,140 @@ public class GroovyScriptEngine implements ResourceConnector {
                     // GROOVY-4013: If it is an inner class, tracking its dependencies doesn't really
                     // serve any purpose and also interferes with the caching done to track dependencies
                     if (classNode instanceof InnerClassNode) return;
-                    DependencyTracker dt = new DependencyTracker(source, cache);
+                    DependencyTracker dt = new DependencyTracker(source, cache, precompiledEntries);
                     dt.visitClass(classNode);
                 }
             }, Phases.CLASS_GENERATION);
+
+            cu.setClassNodeResolver(new ClassNodeResolver() {
+                @Override
+                public LookupResult findClassNode(String origName, CompilationUnit compilationUnit) {
+                    CompilerConfiguration cc = compilationUnit.getConfiguration();
+                    String name = origName.replace('.', '/');
+                    for (String ext : cc.getScriptExtensions()) {
+                        try {
+                            String finalName = name + "." + ext;
+                            URLConnection conn = rc.getResourceConnection(finalName);
+                            URL url = conn.getURL();
+                            String path = url.toExternalForm();
+                            ScriptCacheEntry entry = scriptCache.get(path);
+                            Class clazz = null;
+                            if (entry != null) clazz = entry.scriptClass;
+                            if (GroovyScriptEngine.this.isSourceNewer(entry)) {
+                                try {
+                                    SourceUnit su = compilationUnit.addSource(url);
+                                    return new LookupResult(su, null);
+                                } finally {
+                                    forceClose(conn);
+                                }
+                            } else {
+                                precompiledEntries.put(origName, path);
+                            }
+                            if (clazz != null) {
+                                ClassNode cn = new ClassNode(clazz);
+                                return new LookupResult(null, cn);
+                            }
+                        } catch (ResourceException re) {
+                            // skip
+                        }
+                    }
+                    return super.findClassNode(origName, compilationUnit);
+                }
+            });
+
+            final List<CompilationCustomizer> customizers = config.getCompilationCustomizers();
+            if (customizers != null) {
+                // GROOVY-4813 : apply configuration customizers
+                for (CompilationCustomizer customizer : customizers) {
+                    cu.addPhaseOperation(customizer, customizer.getPhase().getPhaseNumber());
+                }
+            }
+
             return cu;
         }
 
         @Override
         public Class parseClass(GroovyCodeSource codeSource, boolean shouldCacheSource) throws CompilationFailedException {
-            // local is kept as hard reference to avoid garbage collection
-            ThreadLocal<CompilationUnit> localCu = getLocalCompilationUnit();
-            ThreadLocal<StringSetMap> localCache = getDepCache();
+            synchronized (sourceCache) {
+                return doParseClass(codeSource);
+            }
+        }
 
-            // we put the old dependencies into local cache so createCompilationUnit 
+        private Class doParseClass(GroovyCodeSource codeSource) {
+            // local is kept as hard reference to avoid garbage collection
+            ThreadLocal<LocalData> localTh = getLocalData();
+            LocalData localData = new LocalData();
+            localTh.set(localData);
+            StringSetMap cache = localData.dependencyCache;
+
+            // we put the old dependencies into local cache so createCompilationUnit
             // can pick it up. We put that entry under the name "."
             ScriptCacheEntry origEntry = scriptCache.get(codeSource.getName());
             Set<String> origDep = null;
             if (origEntry != null) origDep = origEntry.dependencies;
-            if (origDep != null) localCache.get().put(".", origDep);
+            if (origDep != null) {
+                Set<String> newDep = new HashSet<String>(origDep.size());
+                for (String depName : origDep) {
+                    ScriptCacheEntry dep = scriptCache.get(depName);
+                    try {
+                        if (origEntry == dep || GroovyScriptEngine.this.isSourceNewer(dep)) {
+                            newDep.add(depName);
+                        }
+                    } catch (ResourceException re) {
+
+                    }
+                }
+                cache.put(".", newDep);
+            }
 
             Class answer = super.parseClass(codeSource, false);
 
-            StringSetMap cache = localCache.get();
             cache.makeTransitiveHull();
-            long now = System.currentTimeMillis();
+            long time = getCurrentTime();
             Set<String> entryNames = new HashSet<String>();
             for (Map.Entry<String, Set<String>> entry : cache.entrySet()) {
                 String className = entry.getKey();
                 Class clazz = getClassCacheEntry(className);
                 if (clazz == null) continue;
 
-                String entryName = getPath(clazz);
+                String entryName = getPath(clazz, localData.precompiledEntries);
                 if (entryNames.contains(entryName)) continue;
                 entryNames.add(entryName);
-                Set<String> value = convertToPaths(entry.getValue());
-                ScriptCacheEntry cacheEntry = new ScriptCacheEntry(clazz, now, value);
+                Set<String> value = convertToPaths(entry.getValue(), localData.precompiledEntries);
+                long lastModified;
+                try {
+                    lastModified = getLastModified(entryName);
+                } catch (ResourceException e) {
+                    lastModified = time;
+                }
+                ScriptCacheEntry cacheEntry = new ScriptCacheEntry(clazz, lastModified, time, value, false);
                 scriptCache.put(entryName, cacheEntry);
             }
             cache.clear();
-            localCu.set(null);
+            localTh.set(null);
             return answer;
         }
 
-        private String getPath(Class clazz) {
-            ThreadLocal<CompilationUnit> localCu = getLocalCompilationUnit();
+        private String getPath(Class clazz, Map<String, String> precompiledEntries) {
+            CompilationUnit cu = getLocalData().get().cu;
             String name = clazz.getName();
-            ClassNode classNode = localCu.get().getClassNode(name);
-            return classNode.getModule().getContext().getName();
+            ClassNode classNode = cu.getClassNode(name);
+            if (classNode == null) {
+                // this is a precompiled class!
+                String path = precompiledEntries.get(name);
+                if (path == null) throw new GroovyBugError("Precompiled class " + name + " should be available in precompiled entries map, but was not.");
+                return path;
+            } else {
+                return classNode.getModule().getContext().getName();
+            }
         }
 
-        private Set<String> convertToPaths(Set<String> orig) {
+        private Set<String> convertToPaths(Set<String> orig, Map<String, String> precompiledEntries) {
             Set<String> ret = new HashSet<String>();
             for (String className : orig) {
                 Class clazz = getClassCacheEntry(className);
                 if (clazz == null) continue;
-                ret.add(getPath(clazz));
+                ret.add(getPath(clazz, precompiledEntries));
             }
             return ret;
         }
@@ -237,8 +334,9 @@ public class GroovyScriptEngine implements ResourceConnector {
         String line;
         while (true) {
             System.out.print("groovy> ");
-            if ((line = br.readLine()) == null || line.equals("quit"))
+            if ((line = br.readLine()) == null || line.equals("quit")) {
                 break;
+            }
             try {
                 System.out.println(gse.run(line, new Binding()));
             } catch (Exception e) {
@@ -259,7 +357,7 @@ public class GroovyScriptEngine implements ResourceConnector {
                 if (parentLoader instanceof GroovyClassLoader) {
                     return new ScriptClassLoader((GroovyClassLoader) parentLoader);
                 } else {
-                    return new ScriptClassLoader(parentLoader);
+                    return new ScriptClassLoader(parentLoader, config);
                 }
             }
         });
@@ -282,14 +380,9 @@ public class GroovyScriptEngine implements ResourceConnector {
             URL scriptURL = null;
             try {
                 scriptURL = new URL(root, resourceName);
-                groovyScriptConn = scriptURL.openConnection();
-
-                // Make sure we can open it, if we can't it doesn't exist.
-                // Could be very slow if there are any non-file:// URLs in there
-                groovyScriptConn.getInputStream();
+                groovyScriptConn = openConnection(scriptURL);
 
                 break; // Now this is a bit unusual
-
             } catch (MalformedURLException e) {
                 String message = "Malformed URL: " + root + ", " + resourceName;
                 if (se == null) {
@@ -298,8 +391,7 @@ public class GroovyScriptEngine implements ResourceConnector {
                     se = new ResourceException(message, se);
                 }
             } catch (IOException e1) {
-                groovyScriptConn = null;
-                String message = "Cannot open URL: " + scriptURL;
+                String message = "Cannot open URL: " + root + resourceName;
                 groovyScriptConn = null;
                 if (se == null) {
                     se = new ResourceException(message);
@@ -316,35 +408,47 @@ public class GroovyScriptEngine implements ResourceConnector {
         return groovyScriptConn;
     }
 
+    private static URLConnection openConnection(URL scriptURL) throws IOException {
+        URLConnection urlConnection = scriptURL.openConnection();
+        verifyInputStream(urlConnection);
+
+        return scriptURL.openConnection();
+    }
+
     /**
      * This method closes a {@link URLConnection} by getting its {@link InputStream} and calling the
      * {@link InputStream#close()} method on it. The {@link URLConnection} doesn't have a close() method
      * and relies on garbage collection to close the underlying connection to the file.
      * Relying on garbage collection could lead to the application exhausting the number of files the
      * user is allowed to have open at any one point in time and cause the application to crash
-     * ({@link FileNotFoundException} (Too many open files)).
+     * ({@link java.io.FileNotFoundException} (Too many open files)).
      * Hence the need for this method to explicitly close the underlying connection to the file.
      *
      * @param urlConnection the {@link URLConnection} to be "closed" to close the underlying file descriptors.
      */
-    private void forceClose(URLConnection urlConnection) {
+    private static void forceClose(URLConnection urlConnection) {
         if (urlConnection != null) {
             // We need to get the input stream and close it to force the open
             // file descriptor to be released. Otherwise, we will reach the limit
             // for number of files open at one time.
 
-            InputStream in = null;
             try {
-                in = urlConnection.getInputStream();
+                verifyInputStream(urlConnection);
             } catch (Exception e) {
                 // Do nothing: We were not going to use it anyway.
-            } finally {
-                if (in != null) {
-                    try {
-                        in.close();
-                    } catch (IOException e) {
-                        // Do nothing: Just want to make sure it is closed.
-                    }
+            }
+        }
+    }
+
+    private static void verifyInputStream(URLConnection urlConnection) throws IOException {
+        InputStream in = null;
+        try {
+            in = urlConnection.getInputStream();
+        } finally {
+            if (in != null) {
+                try {
+                    in.close();
+                } catch (IOException ignore) {
                 }
             }
         }
@@ -359,7 +463,7 @@ public class GroovyScriptEngine implements ResourceConnector {
      *              be laid out using their package structure like Java classes
      */
     private GroovyScriptEngine(URL[] roots, ClassLoader parent, ResourceConnector rc) {
-        if (roots == null) roots = new URL[0];
+        if (roots == null) roots = EMPTY_URL_ARRAY;
         this.roots = roots;
         if (rc == null) rc = this;
         this.rc = rc;
@@ -425,18 +529,6 @@ public class GroovyScriptEngine implements ResourceConnector {
         return parentLoader;
     }
 
-    /**
-     * @param parentClassLoader ClassLoader to be used as the parent ClassLoader
-     *                          for scripts executed by the engine
-     * @deprecated
-     */
-    public void setParentClassLoader(ClassLoader parentClassLoader) {
-        throw new DeprecationException(
-                "The method GroovyScriptEngine#setParentClassLoader(ClassLoader) " +
-                        "is no longer supported. Specify a parentLoader in the constructor instead."
-        );
-    }
-
     /**
      * Get the class of the scriptName in question, so that you can instantiate
      * Groovy objects with caching and reloading.
@@ -448,15 +540,16 @@ public class GroovyScriptEngine implements ResourceConnector {
      */
     public Class loadScriptByName(String scriptName) throws ResourceException, ScriptException {
         URLConnection conn = rc.getResourceConnection(scriptName);
-        String path = conn.getURL().getPath();
+        String path = conn.getURL().toExternalForm();
         ScriptCacheEntry entry = scriptCache.get(path);
         Class clazz = null;
         if (entry != null) clazz = entry.scriptClass;
         try {
             if (isSourceNewer(entry)) {
                 try {
-                    String encoding = conn.getContentEncoding() != null ? conn.getContentEncoding() : "UTF-8";
-                    clazz = groovyLoader.parseClass(DefaultGroovyMethods.getText(conn.getInputStream(), encoding), path);
+                    String encoding = conn.getContentEncoding() != null ? conn.getContentEncoding() : config.getSourceEncoding();
+                    String content = IOGroovyMethods.getText(conn.getInputStream(), encoding);
+                    clazz = groovyLoader.parseClass(content, path);
                 } catch (IOException e) {
                     throw new ResourceException(e);
                 }
@@ -467,25 +560,6 @@ public class GroovyScriptEngine implements ResourceConnector {
         return clazz;
     }
 
-    /**
-     * Get the class of the scriptName in question, so that you can instantiate
-     * Groovy objects with caching and reloading.
-     *
-     * @param scriptName        resource name pointing to the script
-     * @param parentClassLoader the class loader to use when loading the script
-     * @return the loaded scriptName as a compiled class
-     * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException   if there is a problem parsing the script
-     * @deprecated
-     */
-    public Class loadScriptByName(String scriptName, ClassLoader parentClassLoader)
-            throws ResourceException, ScriptException {
-        throw new DeprecationException(
-                "The method GroovyScriptEngine#loadScriptByName(String,ClassLoader) " +
-                        "is no longer supported. Use GroovyScriptEngine#loadScriptByName(String) instead."
-        );
-    }
-
     /**
      * Run a script identified by name with a single argument.
      *
@@ -528,29 +602,51 @@ public class GroovyScriptEngine implements ResourceConnector {
         return InvokerHelper.createScript(loadScriptByName(scriptName), binding);
     }
 
-    protected boolean isSourceNewer(ScriptCacheEntry entry) throws ResourceException {
-        if (entry == null) return true;
-        long now = System.currentTimeMillis();
-
-        for (String scriptName : entry.dependencies) {
-            ScriptCacheEntry depEntry = scriptCache.get(scriptName);
-            long nextPossibleRecompilationTime = depEntry.lastModified + config.getMinimumRecompilationInterval();
-            if (nextPossibleRecompilationTime > now) continue;
-
-            URLConnection conn = rc.getResourceConnection(scriptName);
-            // getLastModified() truncates up to 999 ms from the true modification time, let's fix that
-            long lastMod = ((conn.getLastModified() / 1000) + 1) * 1000 - 1;
+    private long getLastModified(String scriptName) throws ResourceException {
+        URLConnection conn = rc.getResourceConnection(scriptName);
+        long lastMod = 0;
+        try {
+            lastMod = conn.getLastModified();
+        } finally {
             // getResourceConnection() opening the inputstream, let's ensure all streams are closed
             forceClose(conn);
+        }
+        return lastMod;
+    }
 
+    protected boolean isSourceNewer(ScriptCacheEntry entry) throws ResourceException {
+        if (entry == null) return true;
+
+        long mainEntryLastCheck = entry.lastCheck;
+        long now = 0;
+
+        boolean returnValue = false;
+        for (String scriptName : entry.dependencies) {
+            ScriptCacheEntry depEntry = scriptCache.get(scriptName);
+            if (depEntry.sourceNewer) return true;
+
+            // check if maybe dependency was recompiled, but this one here not
+            if (mainEntryLastCheck < depEntry.lastModified) {
+                returnValue = true;
+                continue;
+            }
+
+            if (now == 0) now = getCurrentTime();
+            long nextSourceCheck = depEntry.lastCheck + config.getMinimumRecompilationInterval();
+            if (nextSourceCheck > now) continue;
+
+            long lastMod = getLastModified(scriptName);
             if (depEntry.lastModified < lastMod) {
-                ScriptCacheEntry newEntry = new ScriptCacheEntry(depEntry.scriptClass, lastMod, depEntry.dependencies);
-                scriptCache.put(scriptName, newEntry);
-                return true;
+                depEntry = new ScriptCacheEntry(depEntry, lastMod, true);
+                scriptCache.put(scriptName, depEntry);
+                returnValue = true;
+            } else {
+                depEntry = new ScriptCacheEntry(depEntry, now, false);
+                scriptCache.put(scriptName, depEntry);
             }
         }
 
-        return false;
+        return returnValue;
     }
 
     /**
@@ -580,4 +676,8 @@ public class GroovyScriptEngine implements ResourceConnector {
         if (config == null) throw new NullPointerException("configuration cannot be null");
         this.config = config;
     }
+
+    protected long getCurrentTime() {
+        return System.currentTimeMillis();
+    }
 }
