@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2014 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,7 +30,7 @@ import org.codehaus.groovy.antlr.GroovySourceAST;
  * call on the closing visit - c.visitDefault() then b.visitDefault() then a.visitDefault()
  *
  * @author <a href="mailto:groovy@ross-rayner.com">Jeremy Rayner</a>
- * @version $Revision: 7922 $
+ * @version $Revision$
  */
 
 public class CompositeVisitor implements Visitor{
@@ -183,10 +183,10 @@ public class CompositeVisitor implements Visitor{
         while (itr.hasNext()) {((Visitor)itr.next()).visitClosedBlock(t,visit);}
     }
 
-	public void visitClosureList(GroovySourceAST t, int visit) {
+    public void visitClosureList(GroovySourceAST t, int visit) {
         Iterator itr = itr(visit);
         while (itr.hasNext()) {((Visitor)itr.next()).visitClosureList(t,visit);}
-	}
+    }
 
     public void visitClosureOp(GroovySourceAST t, int visit) {
         Iterator itr = itr(visit);
@@ -758,6 +758,16 @@ public class CompositeVisitor implements Visitor{
         while (itr.hasNext()) {((Visitor)itr.next()).visitModAssign(t,visit);}
     }
 
+    public void visitMultiCatch(GroovySourceAST t, int visit) {
+        Iterator itr = itr(visit);
+        while (itr.hasNext()) {((Visitor)itr.next()).visitMultiCatch(t, visit);}
+    }
+
+    public void visitMultiCatchTypes(final GroovySourceAST t, final int visit) {
+        Iterator itr = itr(visit);
+        while (itr.hasNext()) {((Visitor)itr.next()).visitMultiCatchTypes(t, visit);}
+    }
+
     public void visitNls(GroovySourceAST t, int visit) {
         Iterator itr = itr(visit);
         while (itr.hasNext()) {((Visitor)itr.next()).visitNls(t,visit);}
@@ -1043,6 +1053,11 @@ public class CompositeVisitor implements Visitor{
         while (itr.hasNext()) {((Visitor)itr.next()).visitSuperCtorCall(t,visit);}
     }
 
+    public void visitTraitDef(GroovySourceAST t, int visit) {
+        Iterator itr = itr(visit);
+        while (itr.hasNext()) {((Visitor)itr.next()).visitTraitDef(t,visit);}
+    }
+
     public void visitTripleDot(GroovySourceAST t, int visit) {
         Iterator itr = itr(visit);
         while (itr.hasNext()) {((Visitor)itr.next()).visitTripleDot(t,visit);}
