@@ -16,6 +16,8 @@
 
 package org.codehaus.groovy.ast;
 
+import static org.codehaus.groovy.ast.ClassHelper.GROOVY_OBJECT_TYPE;
+
 import org.codehaus.groovy.ast.tools.GenericsUtils;
 import org.codehaus.groovy.ast.tools.WideningCategories;
 
@@ -31,6 +33,8 @@ import java.util.Set;
  * @see ClassNode
  */
 public class GenericsType extends ASTNode {
+    public static final GenericsType[] EMPTY_ARRAY = new GenericsType[0];
+    
     private final ClassNode[] upperBounds;
     private final ClassNode lowerBound;
     private ClassNode type;
@@ -84,13 +88,23 @@ public class GenericsType extends ASTNode {
         return ret;
     }
 
+    private String nameOf(ClassNode theType) {
+        StringBuilder ret = new StringBuilder();
+        if (theType.isArray()) {
+            ret.append(nameOf(theType.getComponentType()));
+            ret.append("[]");
+        } else {
+            ret.append(theType.getName());
+        }
+        return ret.toString();
+    }
+
     private String genericsBounds(ClassNode theType, Set<String> visited) {
 
         StringBuilder ret = new StringBuilder();
 
         if (theType.isArray()) {
-            ret.append(theType.getComponentType().getName());
-            ret.append("[]");
+            ret.append(nameOf(theType));
         } else if (theType.redirect() instanceof InnerClassNode) {
             InnerClassNode innerClassNode = (InnerClassNode) theType.redirect();
             String parentClassNodeName = innerClassNode.getOuterClass().getName();
@@ -194,6 +208,10 @@ public class GenericsType extends ASTNode {
             if (result) {
                 return true;
             }
+            if (GROOVY_OBJECT_TYPE.equals(superOrInterface) && type.getCompileUnit()!=null) {
+                // type is being compiled so it will implement GroovyObject later
+                return true;
+            }
             if (superOrInterface instanceof WideningCategories.LowestUpperBoundClassNode) {
                 WideningCategories.LowestUpperBoundClassNode cn = (WideningCategories.LowestUpperBoundClassNode) superOrInterface;
                 result = implementsInterfaceOrIsSubclassOf(type, cn.getSuperClass());
@@ -230,7 +248,8 @@ public class GenericsType extends ASTNode {
                     if (lowerBound!=null) return genericsTypes[0].getName().equals(lowerBound.getUnresolvedName());
                     if (upperBounds!=null) {
                         for (ClassNode upperBound : upperBounds) {
-                            if (genericsTypes[0].getName().equals(upperBound.getUnresolvedName())) return true;
+                            String name = upperBound.getGenericsTypes()[0].getName();
+                            if (genericsTypes[0].getName().equals(name)) return true;
                         }
                         return false;
                     }
@@ -365,7 +384,7 @@ public class GenericsType extends ASTNode {
                             match = false;
                             if (genericsType!=null) {
                                 if (genericsType.isPlaceholder()) {
-                                    match = genericsType.getName().equals(name);
+                                    match = true;
                                 } else if (genericsType.isWildcard()) {
                                     if (genericsType.getUpperBounds()!=null) {
                                         for (ClassNode up : genericsType.getUpperBounds()) {
@@ -434,7 +453,8 @@ public class GenericsType extends ASTNode {
                             match = redirectBoundType.isCompatibleWith(classNodeType.getType());
                         }
                     } else {
-                        match = classNodeType.isCompatibleWith(redirectBoundType.getType());
+                        // todo: the check for isWildcard should be replaced with a more complete check
+                        match = redirectBoundType.isWildcard() || classNodeType.isCompatibleWith(redirectBoundType.getType());
                     }
                 }
             }
