@@ -65,6 +65,7 @@ public class ListExpression extends Expression {
     public Expression transformExpression(ExpressionTransformer transformer) {
         Expression ret = new ListExpression(transformExpressions(getExpressions(), transformer));
         ret.setSourcePosition(this);
+        ret.copyNodeMetaData(this);
         return ret;
     }
 
@@ -73,7 +74,7 @@ public class ListExpression extends Expression {
     }
 
     public String getText() {
-        StringBuffer buffer = new StringBuffer("[");
+        StringBuilder buffer = new StringBuilder("[");
         boolean first = true;
         for (Expression expression : expressions) {
             if (first) {
