@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,22 +16,33 @@
 
 package org.codehaus.groovy.runtime;
 
+import groovy.lang.GroovyRuntimeException;
+import org.codehaus.groovy.vmplugin.VMPlugin;
+import org.codehaus.groovy.vmplugin.VMPluginFactory;
+
+import java.io.Serializable;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.lang.reflect.Proxy;
+import java.util.concurrent.ConcurrentHashMap;
 
 /**
  * This class is a general adapter to map a call to a Java interface
  * to a given delegate.
- * <p/>
  *
  * @author Ben Yu
  * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
  */
-public abstract class ConversionHandler implements InvocationHandler {
+public abstract class ConversionHandler implements InvocationHandler, Serializable {
     private Object delegate;
-    
+    private static final long serialVersionUID = 1162833717190835227L;
+    private ConcurrentHashMap handleCache;
+    {
+        if (VMPluginFactory.getPlugin().getVersion()>=7) handleCache = new ConcurrentHashMap();
+    }
+
     /**
      * Creates a ConversionHandler with an delegate.
      *
@@ -54,16 +65,20 @@ public abstract class ConversionHandler implements InvocationHandler {
 
     /**
      * This method is a default implementation for the invoke method given in
-     * Invocationhandler. Any call to a method with a declaring class that is
-     * not Object, excluding toString(), is redirected to invokeCustom.
+     * InvocationHandler. Any call to a method with a declaring class that is
+     * not Object, excluding toString() and default methods is redirected to invokeCustom.
+     * <p>
      * Methods like equals and hashcode are called on the class itself instead
      * of the delegate because they are considered fundamental methods that should
      * not be overwritten. The toString() method gets special treatment as it is
      * deemed to be a method that you might wish to override when called from Groovy.
-     *
+     * Interface default methods from Java 8 on the other hand are considered being
+     * default implementations you don't normally want to change. So they are called
+     * directly too
+     * </p><p>
      * In many scenarios, it is better to overwrite the invokeCustom method where
      * the core Object related methods are filtered out.
-     *
+     *</p>
      * @param proxy  the proxy
      * @param method the method
      * @param args   the arguments
@@ -73,9 +88,24 @@ public abstract class ConversionHandler implements InvocationHandler {
      * @see InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method, java.lang.Object[])
      */
     public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
-        if (!checkMethod(method)) {
-            return invokeCustom(proxy, method, args);
+        VMPlugin plugin = VMPluginFactory.getPlugin();
+        if (plugin.getVersion()>=7 && isDefaultMethod(method)) {
+            Object handle = handleCache.get(method);
+            if (handle == null) {
+                handle = plugin.getInvokeSpecialHandle(method, proxy);
+                handleCache.put(method, handle);
+            }
+            return plugin.invokeHandle(handle, args);
         }
+
+        if (!checkMethod(method)) {
+            try {
+                return invokeCustom(proxy, method, args);
+            } catch (GroovyRuntimeException gre) {
+                throw ScriptBytecodeAdapter.unwrap(gre);
+            }
+        }
+
         try {
             return method.invoke(this, args);
         } catch (InvocationTargetException ite) {
@@ -83,8 +113,13 @@ public abstract class ConversionHandler implements InvocationHandler {
         }
     }
 
+    protected boolean isDefaultMethod(Method method) {
+        return ((method.getModifiers() & (Modifier.ABSTRACT | Modifier.PUBLIC | Modifier.STATIC)) ==
+                Modifier.PUBLIC) && method.getDeclaringClass().isInterface();
+    }
+
     protected boolean checkMethod(Method method) {
-    	return isCoreObjectMethod(method);
+        return isCoreObjectMethod(method);
     }
 
     /**
@@ -151,4 +186,5 @@ public abstract class ConversionHandler implements InvocationHandler {
     public static boolean isCoreObjectMethod(Method method) {
         return Object.class.equals(method.getDeclaringClass());
     }
+
 }
