@@ -19,6 +19,10 @@ package org.codehaus.groovy.syntax;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.HashSet;
+import java.util.Set;
+import java.util.Collections;
+import java.util.Collection;
 
 import org.codehaus.groovy.GroovyBugError;
 
@@ -30,7 +34,7 @@ import org.codehaus.groovy.GroovyBugError;
  *  @author <a href="mailto:bob@werken.com">bob mcwhirter</a>
  *  @author <a href="mailto:cpoirier@dreaming.org">Chris Poirier</a>
  *
- *  @version $Id: Types.java 6778 2007-07-02 10:40:33Z glaforge $
+ *  @version $Id$
  */
 
 public class Types
@@ -286,7 +290,7 @@ public class Types
     public static final int GSTRING_START               = 901;   // any marker tha begins a GString
     public static final int GSTRING_END                 = 902;   // any matching marker that ends a GString
     public static final int GSTRING_EXPRESSION_START    = 903;   // the ${ marker that starts a GString expression
-    public static final int GSTRING_EXPRESSION_END      = 904;   // the } marker that ends a GString expresssion
+    public static final int GSTRING_EXPRESSION_END      = 904;   // the } marker that ends a GString expression
 
 
     //
@@ -677,12 +681,12 @@ public class Types
                     return true;
                 }
                 switch( specific )
-				{
-                	case LEFT_PARENTHESIS:
+                {
+                    case LEFT_PARENTHESIS:
                     case GSTRING_START:
                     case SYNTH_GSTRING:
                     case KEYWORD_NEW:
-                    	return true;
+                        return true;
                 }
                 break;
 
@@ -963,7 +967,7 @@ public class Types
 
 
    /**
-    *  Returns the precendence of the specified operator.  Non-operator's will
+    *  Returns the precedence of the specified operator.  Non-operator's will
     *  receive -1 or a GroovyBugError, depending on your preference.
     */
 
@@ -1002,7 +1006,7 @@ public class Types
                 return 20;
 
             case BITWISE_OR:
-	    case BITWISE_AND:
+            case BITWISE_AND:
             case BITWISE_XOR:
                 return 22;
 
@@ -1088,9 +1092,17 @@ public class Types
   //---------------------------------------------------------------------------
   // TEXTS
 
-    private static final Map TEXTS  = new HashMap();  // symbol/keyword type -> text
-    private static final Map LOOKUP = new HashMap();  // text -> symbol/keyword type
+    private static final Map<Integer,String> TEXTS  = new HashMap<Integer,String>();  // symbol/keyword type -> text
+    private static final Map<String,Integer> LOOKUP = new HashMap<String,Integer>();  // text -> symbol/keyword type
+    private static final Set<String> KEYWORDS = new HashSet<String>();  // valid keywords
 
+    public static Collection<String> getKeywords() {
+        return Collections.unmodifiableSet(KEYWORDS);
+    }
+
+    public static boolean isKeyword(final String text) {
+        return KEYWORDS.contains(text);
+    }
 
    /**
     *  Returns the type for the specified symbol/keyword text.  Returns UNKNOWN
@@ -1101,7 +1113,7 @@ public class Types
         int type = UNKNOWN;
 
         if( LOOKUP.containsKey(text) ) {
-            type = ((Integer)LOOKUP.get(text)).intValue();
+            type = LOOKUP.get(text);
             if( filter != UNKNOWN && !ofType(type, filter) ) {
                 type = UNKNOWN;
             }
@@ -1137,28 +1149,31 @@ public class Types
     */
 
     public static String getText( int type ) {
-        Integer key = new Integer( type );
         String text = "";
 
-        if( TEXTS.containsKey(key) ) {
-            text = (String)TEXTS.get( key );
+        if( TEXTS.containsKey(type) ) {
+            text = TEXTS.get( type );
         }
 
         return text;
     }
 
 
-   /**
-    *  Adds a element to the TEXTS and LOOKUP.
-    */
-
+    /**
+     *  Adds a element to the TEXTS and LOOKUP.
+     */
     private static void addTranslation( String text, int type ) {
-        Integer key = new Integer( type );
-
-        TEXTS.put( key, text );
-        LOOKUP.put( text, key );
+        TEXTS.put( type, text );
+        LOOKUP.put( text, type );
     }
 
+    /**
+     *  Adds a element to the KEYWORDS, TEXTS and LOOKUP.
+     */
+    private static void addKeyword( String text, int type ) {
+        KEYWORDS.add(text);
+        addTranslation(text, type);
+    }
 
     static {
 
@@ -1210,8 +1225,7 @@ public class Types
         addTranslation( "\\"          , INTDIV                      );
         addTranslation( "%"           , MOD                         );
 
-	addTranslation( "**"          , POWER                       );
-
+        addTranslation( "**"          , POWER                       );
         addTranslation( "+="          , PLUS_EQUAL                  );
         addTranslation( "-="          , MINUS_EQUAL                 );
         addTranslation( "*="          , MULTIPLY_EQUAL              );
@@ -1250,65 +1264,63 @@ public class Types
         //
         // Keywords
 
-        addTranslation( "abstract"    , KEYWORD_ABSTRACT            );
-        addTranslation( "as"          , KEYWORD_AS                  );
-        addTranslation( "assert"      , KEYWORD_ASSERT              );
-        addTranslation( "break"       , KEYWORD_BREAK               );
-        addTranslation( "case"        , KEYWORD_CASE                );
-        addTranslation( "catch"       , KEYWORD_CATCH               );
-        addTranslation( "class"       , KEYWORD_CLASS               );
-        addTranslation( "const"       , KEYWORD_CONST               );
-        addTranslation( "continue"    , KEYWORD_CONTINUE            );
-        addTranslation( "def"         , KEYWORD_DEF                 );
-        addTranslation( "defmacro"    , KEYWORD_DEF                 ); // xxx br defmacro
-        addTranslation( "default"     , KEYWORD_DEFAULT             );
-        addTranslation( "do"          , KEYWORD_DO                  );
-        addTranslation( "else"        , KEYWORD_ELSE                );
-        addTranslation( "extends"     , KEYWORD_EXTENDS             );
-        addTranslation( "final"       , KEYWORD_FINAL               );
-        addTranslation( "finally"     , KEYWORD_FINALLY             );
-        addTranslation( "for"         , KEYWORD_FOR                 );
-        addTranslation( "goto"        , KEYWORD_GOTO                );
-        addTranslation( "if"          , KEYWORD_IF                  );
-        addTranslation( "in"          , KEYWORD_IN                  );
-        addTranslation( "implements"  , KEYWORD_IMPLEMENTS          );
-        addTranslation( "import"      , KEYWORD_IMPORT              );
-        addTranslation( "instanceof"  , KEYWORD_INSTANCEOF          );
-        addTranslation( "interface"   , KEYWORD_INTERFACE           );
-        addTranslation( "mixin"       , KEYWORD_MIXIN               );
-        addTranslation( "native"      , KEYWORD_NATIVE              );
-        addTranslation( "new"         , KEYWORD_NEW                 );
-        addTranslation( "package"     , KEYWORD_PACKAGE             );
-        addTranslation( "private"     , KEYWORD_PRIVATE             );
-        addTranslation( "property"    , KEYWORD_PROPERTY            );
-        addTranslation( "protected"   , KEYWORD_PROTECTED           );
-        addTranslation( "public"      , KEYWORD_PUBLIC              );
-        addTranslation( "return"      , KEYWORD_RETURN              );
-        addTranslation( "static"      , KEYWORD_STATIC              );
-        addTranslation( "super"       , KEYWORD_SUPER               );
-        addTranslation( "switch"      , KEYWORD_SWITCH              );
-        addTranslation( "synchronized", KEYWORD_SYNCHRONIZED        );
-        addTranslation( "this"        , KEYWORD_THIS                );
-        addTranslation( "throw"       , KEYWORD_THROW               );
-        addTranslation( "throws"      , KEYWORD_THROWS              );
-        addTranslation( "transient"   , KEYWORD_TRANSIENT           );
-        addTranslation( "try"         , KEYWORD_TRY                 );
-        addTranslation( "volatile"    , KEYWORD_VOLATILE            );
-        addTranslation( "while"       , KEYWORD_WHILE               );
-
-        addTranslation( "true"        , KEYWORD_TRUE                );
-        addTranslation( "false"       , KEYWORD_FALSE               );
-        addTranslation( "null"        , KEYWORD_NULL                );
-
-        addTranslation( "void"        , KEYWORD_VOID                );
-        addTranslation( "boolean"     , KEYWORD_BOOLEAN             );
-        addTranslation( "byte"        , KEYWORD_BYTE                );
-        addTranslation( "int"         , KEYWORD_INT                 );
-        addTranslation( "short"       , KEYWORD_SHORT               );
-        addTranslation( "long"        , KEYWORD_LONG                );
-        addTranslation( "float"       , KEYWORD_FLOAT               );
-        addTranslation( "double"      , KEYWORD_DOUBLE              );
-        addTranslation( "char"        , KEYWORD_CHAR                );
+        addKeyword( "abstract"    , KEYWORD_ABSTRACT            );
+        addKeyword( "as"          , KEYWORD_AS                  );
+        addKeyword( "assert"      , KEYWORD_ASSERT              );
+        addKeyword( "break"       , KEYWORD_BREAK               );
+        addKeyword( "case"        , KEYWORD_CASE                );
+        addKeyword( "catch"       , KEYWORD_CATCH               );
+        addKeyword( "class"       , KEYWORD_CLASS               );
+        addKeyword( "const"       , KEYWORD_CONST               );
+        addKeyword( "continue"    , KEYWORD_CONTINUE            );
+        addKeyword( "def"         , KEYWORD_DEF                 );
+        addKeyword( "defmacro"    , KEYWORD_DEF                 ); // xxx br defmacro
+        addKeyword( "default"     , KEYWORD_DEFAULT             );
+        addKeyword( "do"          , KEYWORD_DO                  );
+        addKeyword( "else"        , KEYWORD_ELSE                );
+        addKeyword( "extends"     , KEYWORD_EXTENDS             );
+        addKeyword( "final"       , KEYWORD_FINAL               );
+        addKeyword( "finally"     , KEYWORD_FINALLY             );
+        addKeyword( "for"         , KEYWORD_FOR                 );
+        addKeyword( "goto"        , KEYWORD_GOTO                );
+        addKeyword( "if"          , KEYWORD_IF                  );
+        addKeyword( "in"          , KEYWORD_IN                  );
+        addKeyword( "implements"  , KEYWORD_IMPLEMENTS          );
+        addKeyword( "import"      , KEYWORD_IMPORT              );
+        addKeyword( "instanceof"  , KEYWORD_INSTANCEOF          );
+        addKeyword( "interface"   , KEYWORD_INTERFACE           );
+        addKeyword( "mixin"       , KEYWORD_MIXIN               );
+        addKeyword( "native"      , KEYWORD_NATIVE              );
+        addKeyword( "new"         , KEYWORD_NEW                 );
+        addKeyword( "package"     , KEYWORD_PACKAGE             );
+        addKeyword( "private"     , KEYWORD_PRIVATE             );
+        addKeyword( "property"    , KEYWORD_PROPERTY            );
+        addKeyword( "protected"   , KEYWORD_PROTECTED           );
+        addKeyword( "public"      , KEYWORD_PUBLIC              );
+        addKeyword( "return"      , KEYWORD_RETURN              );
+        addKeyword( "static"      , KEYWORD_STATIC              );
+        addKeyword( "super"       , KEYWORD_SUPER               );
+        addKeyword( "switch"      , KEYWORD_SWITCH              );
+        addKeyword( "synchronized", KEYWORD_SYNCHRONIZED        );
+        addKeyword( "this"        , KEYWORD_THIS                );
+        addKeyword( "throw"       , KEYWORD_THROW               );
+        addKeyword( "throws"      , KEYWORD_THROWS              );
+        addKeyword( "transient"   , KEYWORD_TRANSIENT           );
+        addKeyword( "try"         , KEYWORD_TRY                 );
+        addKeyword( "volatile"    , KEYWORD_VOLATILE            );
+        addKeyword( "while"       , KEYWORD_WHILE               );
+        addKeyword( "true"        , KEYWORD_TRUE                );
+        addKeyword( "false"       , KEYWORD_FALSE               );
+        addKeyword( "null"        , KEYWORD_NULL                );
+        addKeyword( "void"        , KEYWORD_VOID                );
+        addKeyword( "boolean"     , KEYWORD_BOOLEAN             );
+        addKeyword( "byte"        , KEYWORD_BYTE                );
+        addKeyword( "int"         , KEYWORD_INT                 );
+        addKeyword( "short"       , KEYWORD_SHORT               );
+        addKeyword( "long"        , KEYWORD_LONG                );
+        addKeyword( "float"       , KEYWORD_FLOAT               );
+        addKeyword( "double"      , KEYWORD_DOUBLE              );
+        addKeyword( "char"        , KEYWORD_CHAR                );
     }
 
 
@@ -1318,7 +1330,7 @@ public class Types
   // DESCRIPTIONS
 
 
-    private static final Map DESCRIPTIONS = new HashMap();
+    private static final Map<Integer,String> DESCRIPTIONS = new HashMap<Integer,String>();
 
 
    /**
@@ -1326,10 +1338,8 @@ public class Types
     */
 
     public static String getDescription( int type ) {
-        Integer typeKey = new Integer(type);
-
-        if (DESCRIPTIONS.containsKey(typeKey)) {
-            return (String)DESCRIPTIONS.get(typeKey);
+        if (DESCRIPTIONS.containsKey(type)) {
+            return DESCRIPTIONS.get(type);
         }
 
         return "<>";
@@ -1341,15 +1351,6 @@ public class Types
     */
 
     private static void addDescription(int type, String description) {
-        addDescription(new Integer(type), description);
-    }
-
-
-   /**
-    *  Adds a description to the set.
-    */
-
-    private static void addDescription(Integer type, String description) {
         if (description.startsWith("<") && description.endsWith(">")) {
             DESCRIPTIONS.put(type, description);
         }
@@ -1361,12 +1362,11 @@ public class Types
 
     static {
 
-        Iterator iterator = LOOKUP.keySet().iterator();
+        Iterator<String> iterator = LOOKUP.keySet().iterator();
         while( iterator.hasNext() )
         {
-            String text = (String)iterator.next();
-            Integer key = (Integer)LOOKUP.get(text);
-
+            String text = iterator.next();
+            int key = LOOKUP.get(text);
             addDescription( key, text );
         }
 
