@@ -1,3 +1,18 @@
+/**
+ * Copyright 2003-2015 the original author or authors.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 /**
  *
  */
@@ -41,7 +56,7 @@ public class CharacterRangeTest extends TestCase {
      */
     public void testIterate() {
         Iterator iter = range.iterator();
-        assertEquals("wrong first value", FROM, iter.next());
+        assertEquals(FROM, iter.next());
         for (char expected = (char) (FROM.charValue() + 1); expected <= TO.charValue(); expected++) {
             assertEquals(expected, ((Character)iter.next()).charValue());
         }
