@@ -1,3 +1,18 @@
+/**
+ * Copyright 2003-2015 the original author or authors.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package org.codehaus.groovy.ast;
 
 import java.util.List;
@@ -27,7 +42,6 @@ import org.codehaus.groovy.ast.expr.PostfixExpression;
 import org.codehaus.groovy.ast.expr.PrefixExpression;
 import org.codehaus.groovy.ast.expr.PropertyExpression;
 import org.codehaus.groovy.ast.expr.RangeExpression;
-import org.codehaus.groovy.ast.expr.RegexExpression;
 import org.codehaus.groovy.ast.expr.SpreadExpression;
 import org.codehaus.groovy.ast.expr.SpreadMapExpression;
 import org.codehaus.groovy.ast.expr.StaticMethodCallExpression;
@@ -406,12 +420,6 @@ class LineCheckVisitor extends ClassCodeVisitorSupport {
         super.visitFieldExpression(expression);
     }
 
-    @Deprecated
-    public void visitRegexExpression(RegexExpression expression) {
-        visitNode(expression);
-        super.visitRegexExpression(expression);
-    }
-
     public void visitGStringExpression(GStringExpression expression) {
         visitNode(expression);
         super.visitGStringExpression(expression);
